# Copyright (c) 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import logging

from skew.resources.aws import AWSResource


LOG = logging.getLogger(__name__)


class Function(AWSResource):

    @classmethod
    def enumerate(cls, arn, region, account, resource_id=None):
        resources = super(Function, cls).enumerate(arn, region, account,
                                                   resource_id)
        for r in resources:
            r.data['EventSources'] = []
            kwargs = {'FunctionName': r.data['FunctionName']}
            response = r._client.call('list_event_source_mappings', **kwargs)
            for esm in response['EventSourceMappings']:
                r.data['EventSources'].append(esm['EventSourceArn'])
        return resources

    class Meta(object):
        service = 'lambda'
        type = 'function'
        enum_spec = ('list_functions', 'Functions', None)
        detail_spec = None
        id = 'FunctionName'
        filter_name = None
        name = 'FunctionName'
        date = 'LastModified'
        dimension = 'FunctionName'

    @property
    def arn(self):
        return self.data.get('FunctionArn')



# Copyright (c) 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import logging
import jmespath

import skew.awsclient

LOG = logging.getLogger(__name__)


class Resource(object):

    @classmethod
    def enumerate(cls, arn, region, account, resource_id=None, aws_creds=None):
        client = skew.awsclient.get_awsclient(
            cls.Meta.service, region, account, aws_creds=aws_creds)
        kwargs = {}
        do_client_side_filtering = False
        if resource_id and resource_id != '*':
            # If we are looking for a specific resource and the
            # API provides a way to filter on a specific resource
            # id then let's insert the right parameter to do the filtering.
            # If the API does not support that, we will have to filter
            # after we get all of the results.
            filter_name = cls.Meta.filter_name
            if filter_name:
                if cls.Meta.filter_type == 'list':
                    kwargs[filter_name] = [resource_id]
                else:
                    kwargs[filter_name] = resource_id
            else:
                do_client_side_filtering = True
        enum_op, path, extra_args = cls.Meta.enum_spec
        if extra_args:
            kwargs.update(extra_args)
        LOG.debug('enum_op=%s' % enum_op)
        data = client.call(enum_op, query=path, **kwargs)
        LOG.debug(data)
        resources = []
        if data:
            for d in data:
                if do_client_side_filtering:
                    # If the API does not support filtering, the resource
                    # class should provide a filter method that will
                    # return True if the returned data matches the
                    # resource ID we are looking for.
                    if not cls.filter(resource_id, d):
                        continue
                resources.append(cls(client, d, arn.query))
        return resources

    class Meta(object):
        type = 'resource'
        dimension = None
        tags_spec = None
        id = None
        date = None
        name = None

    def __init__(self, client, data):
        self._client = client
        if data is None:
            data = {}
        self.data = data
        if hasattr(self.Meta, 'id') and isinstance(self.data, dict):
            self._id = self.data.get(self.Meta.id, '')
        else:
            self._id = ''
        self._metrics = list()
        self._name = None
        self._date = None

    def __repr__(self):
        return self.arn

    @property
    def arn(self):
        return 'arn:aws:%s:%s:%s:%s/%s' % (
            self._client.service_name,
            self._client.region_name,
            self._client.account_id,
            self.resourcetype, self.id)

    @property
    def resourcetype(self):
        return self.Meta.type

    @property
    def parent(self):
        pass

    @property
    def name(self):
        if not self._name:
            self._name = jmespath.search(self.Meta.name, self.data)
        return self._name

    @property
    def id(self):
        return self._id

    @property
    def date(self):
        if not self._date:
            self._date = jmespath.search(self.Meta.date, self.data)
        return self._date

    @property
    def metrics(self):
        if self._metrics is None:
            self._metrics = []
        return self._metrics

    @property
    def metric_names(self):
        return [m['MetricName'] for m in self.metrics]

    def find_metric(self, metric_name):
        for m in self.metrics:
            if m['MetricName'] == metric_name:
                return m
        return None



# Copyright 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import json
import os

import datetime
import jmespath
import botocore.session

from skew.config import get_config

LOG = logging.getLogger(__name__)


def json_encoder(obj):
    """JSON encoder that formats datetimes as ISO8601 format."""
    if isinstance(obj, datetime.datetime):
        return obj.isoformat()
    else:
        return obj


class AWSClient(object):

    def __init__(self, service_name, region_name, account_id, aws_creds=None):
        self._config = get_config()
        self._service_name = service_name
        self._region_name = region_name
        self._account_id = account_id
        self._has_credentials = False
        if not aws_creds:
            # If no creds, need profile name to retrieve creds from ~/.aws/credentials
            self._profile = self._config['accounts'][account_id]['profile']
        self.aws_creds = aws_creds
        self._client = self._create_client()
        self._record_path = self._config.get('record_path', None)

    @property
    def service_name(self):
        return self._service_name

    @property
    def region_name(self):
        return self._region_name

    @property
    def account_id(self):
        return self._account_id

    @property
    def profile(self):
        return self._profile

    def _record(self, op_name, kwargs, data):
        """
        This is a little hack to enable easier unit testing of the code.
        Since botocore has its own set of tests, I'm not interested in
        trying to test it again here.  So, this recording capability allows
        us to save the data coming back from botocore as JSON files which
        can then be used by the mocked awsclient in the unit test directory.
        To enable this, add something like this to your skew config file:

              record_path: ~/projects/skew/skew/tests/unit/data

        and the JSON data files will get stored in this path.
        """
        if self._record_path:
            path = os.path.expanduser(self._record_path)
            path = os.path.expandvars(path)
            path = os.path.join(path, self.service_name)
            if not os.path.isdir(path):
                os.mkdir(path)
            path = os.path.join(path, self.region_name)
            if not os.path.isdir(path):
                os.mkdir(path)
            path = os.path.join(path, self.account_id)
            if not os.path.isdir(path):
                os.mkdir(path)
            filename = op_name
            if kwargs:
                for k, v in kwargs.items():
                    if k != 'query':
                        filename += '_{}_{}'.format(k, v)
            filename += '.json'
            path = os.path.join(path, filename)
            with open(path, 'wb') as fp:
                json.dump(data, fp, indent=4, default=json_encoder,
                          ensure_ascii=False)

    def _create_client(self):
        session = botocore.session.get_session()
        # Try using "local" creds first:
        if self.aws_creds:
            session.set_credentials(**self.aws_creds)
        else:
            session.set_config_variable('profile', self.profile)
        return session.create_client(
            self.service_name, region_name=self.region_name)

    def call(self, op_name, query=None, **kwargs):
        """
        Make a request to a method in this client.  The response data is
        returned from this call as native Python data structures.

        This method differs from just calling the client method directly
        in the following ways:

          * It automatically handles the pagination rather than
            relying on a separate pagination method call.
          * You can pass an optional jmespath query and this query
            will be applied to the data returned from the low-level
            call.  This allows you to tailor the returned data to be
            exactly what you want.

        :type op_name: str
        :param op_name: The name of the request you wish to make.

        :type query: str
        :param query: A jmespath query that will be applied to the
            data returned by the operation prior to returning
            it to the user.

        :type kwargs: keyword arguments
        :param kwargs: Additional keyword arguments you want to pass
            to the method when making the request.
        """
        LOG.debug(kwargs)
        if query:
            query = jmespath.compile(query)
        if self._client.can_paginate(op_name):
            paginator = self._client.get_paginator(op_name)
            results = paginator.paginate(**kwargs)
            data = results.build_full_result()
        else:
            op = getattr(self._client, op_name)
            data = op(**kwargs)
        if query:
            data = query.search(data)
        self._record(op_name, kwargs, data)
        return data


def get_awsclient(service_name, region_name, account_id):
    return AWSClient(service_name, region_name, account_id)



# Copyright (c) 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from skew.resources.aws import AWSResource


class Stream(AWSResource):

    class Meta(object):
        service = 'kinesis'
        type = 'stream'
        enum_spec = ('list_streams', 'StreamNames', None)
        detail_spec = None
        id = 'StreamName'
        filter_name = None
        filter_type = None
        name = 'StreamName'
        date = None
        dimension = 'StreamName'

    def __init__(self, client, data, query=None):
        super(Stream, self).__init__(client, data, query)
        self.data = {self.Meta.id: data}
        self._id = self.data[self.Meta.id]



# Copyright 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import logging

import yaml

from skew.exception import ConfigNotFoundError

LOG = logging.getLogger(__name__)


_config = None


def get_config():
    global _config
    if _config is None:
        path = os.environ.get('SKEW_CONFIG', os.path.join('~', '.skew'))
        path = os.path.expanduser(path)
        path = os.path.expandvars(path)
        if not os.path.exists(path):
            raise ConfigNotFoundError('Unable to find skew config file')
        with open(path) as config_file:
            _config = yaml.load(config_file)
    return _config



# Copyright 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class ConfigNotFoundError(Exception):

    pass


class BaseOperationError(Exception):

    def __init__(self, error_code, error_body, operation_name):
        msg = 'Error(%d) when calling (%s): %s' % (error_code,
                                                   operation_name,
                                                   error_body)
        super(BaseOperationError, self).__init__(msg)
        self.error_code = error_code
        self.error_body = error_body
        self.operation_name = operation_name


class ClientError(BaseOperationError):
    pass


class ServerError(BaseOperationError):
    pass



# Copyright (c) 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from skew.resources.aws import AWSResource


class Route53Resource(AWSResource):

    @property
    def arn(self):
        return 'arn:aws:%s:::%s/%s' % (
            self._client.service_name, self.resourcetype, self.id)


class HostedZone(Route53Resource):

    class Meta(object):
        service = 'route53'
        type = 'hostedzone'
        enum_spec = ('list_hosted_zones', 'HostedZones', None)
        detail_spec = ('GetHostedZone', 'Id', None)
        id = 'Id'
        filter_name = None
        name = 'Name'
        date = None
        dimension = None

    @property
    def id(self):
        return self._id.split('/')[-1]


class HealthCheck(Route53Resource):

    class Meta(object):
        service = 'route53'
        type = 'healthcheck'
        enum_spec = ('list_health_checks', 'HealthChecks', None)
        detail_spec = ('GetHealthCheck', 'Id', None)
        id = 'Id'
        filter_name = None
        name = None
        date = None
        dimension = None


class ResourceRecordSet(Route53Resource):

    class Meta(object):
        service = 'route53'
        type = 'rrset'
        enum_spec = ('list_resource_record_sets', 'ResourceRecordSets', None)
        detail_spec = None
        id = 'Name'
        filter_name = None
        name = 'Name'
        date = None
        dimension = None



# Copyright (c) 2014 Scopely, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import jmespath

from skew.resources.aws import AWSResource


class Stack(AWSResource):

    @classmethod
    def enumerate(cls, arn, region, account, resource_id=None, aws_creds=None):
        resources = super(Stack, cls).enumerate(arn, region, account,
                                                resource_id, aws_creds)
        for stack in resources:
            stack.data['Resources'] = []
            for stack_resource in stack:
                stack.data['Resources'].append(
                    {
                        'id': stack_resource['PhysicalResourceId'],
                        'type': stack_resource['ResourceType']
                    }
                )
        return resources

    class Meta(object):
        service = 'cloudformation'
        type = 'stack'
        enum_spec = ('describe_stacks', 'Stacks[]', None)
        detail_spec = ('describe_stack_resources', 'StackName',
                       'StackResources[]')
        id = 'StackName'
        filter_name = 'StackName'
        name = 'StackName'
        date = 'CreationTime'
        dimension = None

    def __init__(self, client, data, query=None):
        super(Stack, self).__init__(client, data, query)
        self._data = data
        self._resources = []

    def __iter__(self):
        detail_op, param_name, detail_path = self.Meta.detail_spec
        params = {param_name: self.id}
        if not self._resources:
            data = self._client.call(detail_op, **params)
            self._resources = jmespath.search(detail_path, data)
        for resource in self._resources:
            yield resource

    @property
    def arn(self):
        return self._data['StackId']



# Copyright (c) 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import importlib

# Maps resources names as they appear in ARN's to the path name
# of the Python class representing that resource.
ResourceTypes = {
    'aws.autoscaling.autoScalingGroup': 'aws.autoscaling.AutoScalingGroup',
    'aws.autoscaling.launchConfigurationName': 'aws.autoscaling.LaunchConfiguration',
    'aws.cloudformation.stack': 'aws.cloudformation.Stack',
    'aws.cloudwatch.alarm': 'aws.cloudwatch.Alarm',
    'aws.dynamodb.table': 'aws.dynamodb.Table',
    'aws.ec2.address': 'aws.ec2.Address',
    'aws.ec2.customer-gateway': 'aws.ec2.CustomerGateway',
    'aws.ec2.key-pair': 'aws.ec2.KeyPair',
    'aws.ec2.image': 'aws.ec2.Image',
    'aws.ec2.instance': 'aws.ec2.Instance',
    'aws.ec2.network-acl': 'aws.ec2.NetworkAcl',
    'aws.ec2.route-table': 'aws.ec2.RouteTable',
    'aws.ec2.security-group': 'aws.ec2.SecurityGroup',
    'aws.ec2.snapshot': 'aws.ec2.Snapshot',
    'aws.ec2.volume': 'aws.ec2.Volume',
    'aws.ec2.vpc': 'aws.ec2.Vpc',
    'aws.ec2.vpc-peering-connection': 'aws.ec2.VpcPeeringConnection',
    'aws.ec2.subnet': 'aws.ec2.Subnet',
    'aws.elasticache.cluster': 'aws.elasticache.Cluster',
    'aws.elasticache.subnet-group': 'aws.elasticache.SubnetGroup',
    'aws.elasticache.snapshot': 'aws.elasticache.Snapshot',
    'aws.elb.loadbalancer': 'aws.elb.LoadBalancer',
    'aws.iam.group': 'aws.iam.Group',
    'aws.iam.instance-profile': 'aws.iam.InstanceProfile',
    'aws.iam.role': 'aws.iam.Role',
    'aws.iam.policy': 'aws.iam.Policy',
    'aws.iam.user': 'aws.iam.User',
    'aws.iam.server-certificate': 'aws.iam.ServerCertificate',
    'aws.kinesis.stream': 'aws.kinesis.Stream',
    'aws.lambda.function': 'aws.lambda.Function',
    'aws.sqs.queue': 'aws.sqs.Queue',
    'aws.rds.db': 'aws.rds.DBInstance',
    'aws.rds.secgrp': 'aws.rds.DBSecurityGroup',
    'aws.redshift.cluster': 'aws.redshift.Cluster',
    'aws.route53.hostedzone': 'aws.route53.HostedZone',
    'aws.route53.healthcheck': 'aws.route53.HealthCheck',
    'aws.s3.bucket': 'aws.s3.Bucket'
}


def all_providers():
    providers = set()
    for resource_type in ResourceTypes:
        providers.add(resource_type.split('.')[0])
    return list(providers)


def all_services(provider_name):
    services = set()
    for resource_type in ResourceTypes:
        t = resource_type.split('.')
        if t[0] == provider_name:
            services.add(t[1])
    return list(services)


def all_types(provider_name, service_name):
    types = set()
    for resource_type in ResourceTypes:
        t = resource_type.split('.')
        if t[0] == provider_name and t[1] == service_name:
            types.add(t[2])
    return list(types)


def find_resource_class(resource_path):
    """
    dynamically load a class from a string
    """
    class_path = ResourceTypes[resource_path]
    # First prepend our __name__ to the resource string passed in.
    full_path = '.'.join([__name__, class_path])
    class_data = full_path.split(".")
    module_path = ".".join(class_data[:-1])
    class_str = class_data[-1]
    module = importlib.import_module(module_path)
    # Finally, we retrieve the Class
    return getattr(module, class_str)



#!/usr/bin/env python

from setuptools import setup, find_packages

import os

requires = [
    'six>=1.8.0,<2.0.0',
    'botocore>=1.3.4',
    'python-dateutil>=2.1,<3.0.0',
    'PyYAML==3.11']


here = os.path.dirname(os.path.realpath(__file__))
with open(os.path.join(here, 'README.rst')) as readme:
    long_description = readme.read()


setup(
    name='skew',
    version=open(os.path.join('skew', '_version')).read().strip(),
    description='A flat address space for all your AWS resources.',
    long_description=long_description,
    author='Mitch Garnaat',
    author_email='mitch@garnaat.com',
    url='https://github.com/scopely-devops/skew',
    packages=find_packages(exclude=['tests*']),
    package_data={'skew': ['_version']},
    package_dir={'skew': 'skew'},
    install_requires=requires,
    license=open("LICENSE").read(),
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4'
    ),
)



# Copyright (c) 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from skew.resources.aws import AWSResource


class Cluster(AWSResource):

    class Meta(object):
        service = 'elasticache'
        type = 'cluster'
        enum_spec = ('describe_cache_clusters',
                     'CacheClusters[]', None)
        detail_spec = None
        id = 'CacheClusterId'
        filter_name = 'CacheClusterId'
        filter_type = 'scalar'
        name = 'CacheClusterId'
        date = 'CacheClusterCreateTime'
        dimension = 'CacheClusterId'


class SubnetGroup(AWSResource):

    class Meta(object):
        service = 'elasticache'
        type = 'subnet-group'
        enum_spec = ('describe_cache_subnet_groups',
                     'CacheSubnetGroups', None)
        detail_spec = None
        id = 'CacheSubnetGroupName'
        filter_name = 'CacheSubnetGroupName'
        filter_type = 'scalar'
        name = 'CacheSubnetGroupName'
        date = None
        dimension = None


class Snapshot(AWSResource):

    class Meta(object):
        service = 'elasticache'
        type = 'snapshot'
        enum_spec = ('describe_snapshots', 'Snapshots', None)
        detail_spec = None
        id = 'SnapshotName'
        filter_name = 'SnapshotName'
        filter_type = 'scalar'
        name = 'SnapshotName'
        date = 'StartTime'
        dimension = None



# Copyright (c) 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import logging

import jmespath

from skew.resources.aws import AWSResource


LOG = logging.getLogger(__name__)


class Cluster(AWSResource):

    class Meta(object):
        service = 'ecs'
        type = 'cluster'
        enum_spec = ('list_clusters', 'clusterArns', None)
        detail_spec = ('describe_clusters', 'clusters', 'clusters')
        id = 'clusterArn'
        filter_name = 'clusters'
        filter_type = 'list'
        name = 'clusterName'
        date = None
        dimension = 'clusterName'

    def __init__(self, client, data, query=None):
        super(Cluster, self).__init__(client, data, query)
        _, _, _, _, _, type_and_id = data.split(':')
        _, self._id = type_and_id.split('/')
        detail_op, param_name, detail_path = self.Meta.detail_spec
        params = {param_name: [self.id]}
        data = client.call(detail_op, **params)
        self.data = jmespath.search(detail_path, data)


class Service(AWSResource):

    class Meta(object):
        service = 'ecs'
        type = 'service'
        enum_spec = ('list_services', 'serviceArns', None)
        detail_spec = ('describe_services', 'services', 'services')
        id = 'serviceArn'
        filter_name = 'services'
        filter_type = 'list'
        name = 'serviceName'
        date = None
        dimension = 'serviceName'

    def __init__(self, client, data, query=None):
        super(Cluster, self).__init__(client, data, query)
        _, _, _, _, _, type_and_id = data.split(':')
        _, self._id = type_and_id.split('/')
        params = {'cluster': self.name}
        detail_op, param_name, detail_path = self.Meta.detail_spec
        params = {param_name: [self.id]}
        data = client.call(detail_op, **params)
        self.data = jmespath.search(detail_path, data)
